// DE_method.h: interface for the CDE_method class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DE_METHOD_H__EE4A4AE3_1B6C_49FD_AB81_1EEB1556D24C__INCLUDED_)
#define AFX_DE_METHOD_H__EE4A4AE3_1B6C_49FD_AB81_1EEB1556D24C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "global.h"

#include "./EACommonClass/Individual.h"
#include "./EACommonClass/Rand.h"
#include "./EACommonClass/ProblemDef.h"


#include <direct.h>				// include the mkdir() for folder creation

#define out_internal	30		// Ƶ

/* Ӳ̣ԲҪһֽһֽڸ */
#define SIZEOFBUFFER	256*1024L /* СĬΪ256KB */


// Structure of velocity of PSO
struct velocity
{
	int    size;
	double v[N_of_x];
};


typedef struct strct_rewards {
	tFitness reward[500];
	long	 size;
}strct_reward;


class CDE_method  
{
public:
	CDE_method();
	virtual ~CDE_method();

	/* ------------------------class methods------------------------ */
public:
	// public methods
	/* initialize the decision variables of different test problems */
	void init_variables();							// initialize the decision variables
	void init_normal_variables();					// initialize the normal functions' variables
	
	/* initialize the population */
	void init_pop();								// initialize the population
	void init_pop_random();							// initialize the population randomly

	/* evaluate the individual and population */
	void evaluate_ind(CIndividual &);				// evaluate the individual
	void evaluate_normal_ind(CIndividual &);		// evaluate the individual of normal functions

	void evaluate_pop(CIndividual *, int);			// evaluate the population

	/* execute the algorithms */
	void Run_Optimizer(int, double, int, int);		// execute the optimizer

	/* compare the two individuals */
	int  compare_ind(CIndividual *, CIndividual *);	// compare the two individuals
	void find_best_index(CIndividual *, int);		// find the index of the best individual in the population

	void restart_pop(CIndividual *, int);			// restart the population
	void sort_pop(CIndividual *, int);				// sort the population w.r.t fitness with ascend order

	/* some complementary methods */
	void random_index(int *array_index, int all_size, int size);

	void shell_sort_pop(CIndividual* ,int *, int);		// shell sort routine in ascending order
	void shell_sort_array(double *, int *, int);
	void shell_sort_array_t(tFitness *, int *, int);

	inline  int round(double x) { return int(x>0.0?(x+0.5):(x-0.5)); } // round function

	double median(double *array, int size);			// get the median value of the specific array
	double mean_std(double *array, int size, double &stdv);	// get the mean and standard values of the specific array
	tFitness mean_std_t(tFitness *array, int size, tFitness &stdv);
	void   statistic_pop_fitness(CIndividual *pop, int size);

	void display_result(long int);						// display the results in the screen
	void report_result(long int, ofstream &);			// report the results in the files
	void report_diversity(long int, ofstream &);			// report the diversity in the files
	void report_parameter(long int gen, ofstream &file);

	/************************************************************************/
	/* Methods and attributes for batch execution (ִг)      */
	/************************************************************************/
public:
	void	create_folders(int method_ii, int strategy_ii);		// create the folders
	void	copy_files(char *source, char *target);				// copy files from the source directory to the target directory
	long	filesize(FILE *stream);
	int		copyfile(const char* src,const char* dest);

	/* ------------------------class members------------------------ */	
public:
	// public attributes
	CIndividual *parent_pop;				// the main population
	CIndividual *child_pop;					// the child population
	CIndividual *child_pop1;				// the child population used in BBO_DE
	CIndividual best_individual;			// the current best individual

	CIndividual	mutant_vector;				// save the mutant vector
	CIndividual new_child;					// the new child generated by the genetic operator
	
	int pop_size;							// size of the main population
	int best_index;							// index of the current best individual
	int worst_index;

	int flag_precision;						// stop criterion

	long int max_NFFEs;						// maximal number of fitness function evaluations
	long int max_iterations;				// maximal number of iterations
	long int evaluations;					// current number of fitness function evaluations
	long int gen;							// current generation counter
	int MINIMIZE;

	double *upper_bounds;					// upper bounds of the decision variables
	double *lower_bounds;					// lower bounds of the decision variables
	double known_optimal;					// known optima of the problems

	double pop_diversity;
	double max_interval;

	/* some other class objectives */
	double rnd_seed;
	CRand  m_rnd;

	int func_flag;							// flag of the function optimized
	CProblemDef	m_func;						// normal test functions

	int method_flag;

	int	*sort_index;						// only for sorting the population

	/************************************************************************/
	/* Methods and attributes for DE algorithm                              */
	/************************************************************************/
public:
	double	m_F;									// scale factor
	double	m_CR;									// probability of DE crossover
	int		m_strategy;								// mutation strategy
	int		param_adaptation;						// for parameter adaptation used in jDE

	/************************************************************************/
	/* DE with rank-based mutation operators                                */
	/************************************************************************/
public:
	int		weights_flag;
	
	void	calculate_weights(tFitness *S_reward, int S_size, double *weights, int flag);
	void	rank_vector_selection(int base_index, int *index, int size, const double *probability);

	/************************************************************************/
	/* Methods and attributes for AOS-based DE algorithm                    */
	/************************************************************************/
public:
	int		strategy_num;						// number of strategies in the strategy pool

	tFitness Reward[10];						// reward vector
	tFitness Quality[10];						// quality vector
	double	Probability[10];					// probability vector

	int		strategy_index[population_size];

	double	p_min;								// the minimal probability
	double	alpha;								// adaptation rate in [0,1]
	double	beta;								// parameter used in adaptive pursuit

	int		flag_PM_AP;							// PM or AP AOS method (1: PM; 2: AP)

	void	initial_AOS_parameter();			// initialize the AOS parameters
	void	update_quality();					// update the quality vector
	void	update_probability_PM();			// update the PM probability vector
	void	update_probability_AP();			// update the AP probability vector
	int		select_strategy_prob();				// select the strategy based on the probability

	int          window_size;					// the size of the window
	strct_reward reward_array[10];
	tFitness     maxReward;						// the maximal reward, for normalization
	int			 normalized;					// normalize the reward or not
	
	void	insert_reward(int, tFitness);		// insert the reward into the array
	void	get_reward();
	tFitness get_avg_reward(int strategyIndex);
	void	normalize_reward();
	
	void	run_rank_AOS_DE_method();
};

#endif // !defined(AFX_DE_METHOD_H__EE4A4AE3_1B6C_49FD_AB81_1EEB1556D24C__INCLUDED_)
